package com.gem.media.ad;

public interface AdStatusCallBack {

    void onAdLoad(String type, String ad_type, String reason, long time);
    void onAdLoaded(String type, String ad_type, String reason, long time);
    void onNoAdError(String type, String ad_type, String reason, long time);
    void onAdShow(String type, String ad_type, String reason, long time);
    void onAdClick(String type, String ad_type, String reason, long time);
    void onAdDismiss(String type, String ad_type, String reason, long time);
    void onAdLoadTimeout(String type, String ad_type, String reason, long time);
}
